@@ -28,6 +28,7 @@
 
 # $data is NOT a standard member, but if it's there, we'll use it
 # Otherwise, we need to provide data or its name in the call
+#' @exportS3Method recover_data averaging
 recover_data.averaging = function(object, data, ...) {
     ml = attr(object, "modelList")
     if (is.null(ml))
@@ -41,12 +42,13 @@ recover_data.averaging = function(object, data, ...) {
     if (is.null(data))
         data = ml[[1]]$call$data
     ### temporary patch -- $formula often excludes the intercept
-    object$formula = update(object$formula, .~.+1)
+    object$formula = update(object$formula, .~.+1)[-2]
     trms = attr(model.frame(object$formula, data = data), "terms")
     fcall = call("model.avg", formula = object$formula, data = data)
-    recover_data(fcall, delete.response(trms), na.action = NULL, ...)
+    recover_data(fcall, trms, na.action = NULL, ...)
 }
 
+#' @exportS3Method emm_basis averaging
 emm_basis.averaging = function(object, trms, xlev, grid, ...) {
     bhat = coef(object, full = TRUE)
     # change names like "cond(xyz)" to "xyz"
@@ -54,22 +56,33 @@ emm_basis.averaging = function(object, trms, xlev, grid, ...) {
     bnms[bnms == "(Int)"] = "(Intercept)"
     names(bhat) = bnms
     
-    m = suppressWarnings(model.frame(trms, grid, na.action = na.pass, xlev = xlev))
-    X = model.matrix(trms, m, contrasts.arg = object$contrasts)
+    # we're gonna assemble all the main-effects model-matrix components
+    # and build the model matrix to match the names of the coefs
+    vars = .all.vars(trms)
+    mmlist = lapply(vars, \(v) {
+        frm = reformulate(c(-1, v))
+        suppressWarnings(model.matrix(frm, data = grid, xlev = xlev))
+    })
+    ME = cbind("(Intercept)" = 1, do.call(cbind, mmlist))
+    cols = strsplit(names(bhat), ":")  # the cols we need
+    ext = setdiff(unlist(cols), colnames(ME)) # extra expressions we need to compute
+    if(length(ext) > 0) {
+        xcols = sapply(ext, \(expr) eval(parse(text = expr), envir = grid))
+        ME = cbind(ME, xcols)
+    }
+    # now get the factors for each coef
+    X = sapply(cols, \(nms)
+        apply(ME[, nms, drop = FALSE], 1, prod))
+    colnames(X) = names(bhat)
+    # Now, sometimes a coefficient is duplicated, so we need to zero those out
+    srt = sapply(cols, \(nms) paste(sort(nms), collapse = ":"))
+    tmp = table(srt)
+    for (nm in names(tmp[tmp > 1])) {  # work on the dupes
+        zap = which(srt == nm)[-1]   # indices of all but the first
+        bhat[zap] = 0
+    }
     
-    # perm = match(colnames(X), bnms) # fails when order of factors differs
-    # Let's try matching by factors included
-    xlst = lapply(strsplit(colnames(X), ":"), sort)
-    blst = lapply(strsplit(bnms, ":"), sort)
-    matches = lapply(xlst, function(x) which(sapply(blst, \(y) all(x == y))))
-    perm = sapply(matches, function(x) ifelse(length(x) > 0, x[1], NA))
-    
-    missing.terms = colnames(X)[is.na(perm)]
-    if (length(missing.terms) > 0)
-        stop ("In emm_basis.averaging: Unable to match model term(s):\n",
-              paste(missing.terms, collapse = ", "), call. = FALSE)
-    bhat = bhat[perm]
-    V = .my.vcov(object, function(., ...) vcov(., full = TRUE), ...)[perm, perm]
+    V = .my.vcov(object, function(., ...) vcov(., full = TRUE), ...) ###[perm, perm]
     
     nbasis = estimability::all.estble
     ml = attr(object, "modelList")
@@ -95,6 +108,7 @@ emm_basis.averaging = function(object, trms, xlev, grid, ...) {
 ### minc::mira support -----------------------------------------
 # Here we rely on the methods already in place for elements of $analyses
 
+#' @exportS3Method recover_data mira
 recover_data.mira = function(object, data = NULL, ...) {
     rdlist = lapply(object$analyses, recover_data, data = data, ...)
     rd = rdlist[[1]]
@@ -105,6 +119,7 @@ recover_data.mira = function(object, data = NULL, ...) {
     rd
 }
 
+#' @exportS3Method emm_basis mira         
 emm_basis.mira = function(object, trms, xlev, grid, ...) {
     # In case our method did a "pass it on" with the data, we need to add that attribute
     data = list(...)$misc$data
