@@ -802,32 +802,37 @@ ref_grid <- function(object, at, cov.reduce = mean, cov.keep = get_emm_option("c
                 ref.levels[[nm]] = round(ref.levels[[nm]], 3)
             }
             # get only "legal" levels
-            at[[nm]] = at[[nm]][at[[nm]] %in% ref.levels[[nm]]]
-            # Now which of those are left out?
-            excl = setdiff(ref.levels[[nm]], at[[nm]])
-            for (x in excl)
-                incl.flags[grid[[nm]] == x] = FALSE
-            ref.levels[[nm]] = at[[nm]]
+            at[[nm]] = ref.levels[[nm]] = at[[nm]][at[[nm]] %in% ref.levels[[nm]]]
+            rows = numeric(0)
+            for(x in at[[nm]])
+                rows = c(rows, which(grid[[nm]] == x))
+            grid = grid[rows, , drop = FALSE]
+            grid[[nm]] = factor(grid[[nm]], levels = at[[nm]])
+            basis$X = basis$X[rows, , drop = FALSE]
         }
-        if (!any(incl.flags))
-            stop("Reference grid is empty due to mismatched levels in 'at'")
-        grid = grid[incl.flags, , drop=FALSE]
-        basis$X = basis$X[incl.flags, , drop=FALSE]
     }
 
     # Any offsets??? (misc$offset.mult might specify removing or reversing the offset)
     om = ifelse(is.null(misc$offset.mult), 1, misc$offset.mult)
+    oval = 0
     if (!missing(offset)) {  # For safety, we always treat it as scalar
         if (offset[1] != 0)
-            grid[[".offset."]] = offset[1]
+            oval = offset[1]
     }
-    else if(".offset." %in% names(grid))
-        grid[[".offset."]] = om * grid[[".offset."]]
-    else if(!is.null(attr(trms,"offset"))) {
-        if (any(om != 0))
-            grid[[".offset."]] = om * .get.offset(trms, grid)
+    else {
+        if(".static.offset." %in% names(grid)) { # static offset is available
+            oval = om * grid[[".static.offset."]]
+            # grid$.static.offset. = NULL
+        }
+        #else 
+        if(!is.null(attr(trms,"offset"))) {
+            if (any(om != 0))
+                oval = om * (oval + .get.offset(trms, grid))
+        }
+        if(any(oval != 0))
+            grid[[".offset."]] = oval
     }
-
+    
     ### --- Determine weights for each grid point
     if (!hasName(data, "(weights)"))
         data[["(weights)"]] = 1
